# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Commands for reading and manipulating subnetworks."""


from googlecloudsdk.calliope import base


class Subnetworks(base.Group):
  """List, describe, and delete, and update Compute Engine subnetworks."""

Subnetworks.detailed_help = {
    'DESCRIPTION': """
        Read and manipulate VPC subnetworks.

        For more information about VPC, see the
        [VPC documentation](https://cloud.google.com/vpc/docs/vpc).

        See also: [Subnetworks API](https://cloud.google.com/compute/docs/reference/rest/v1/subnetworks).
    """,
}
