# -*- coding: utf-8 -*- #
# Copyright 2024 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command group for Submitting Spark Applciations on Dataproc GDC service instances."""


from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base


@base.Hidden
@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class SparkApplications(base.Group):
  """Submit Dataproc GDC Spark Applications."""

  category = base.DATA_ANALYTICS_CATEGORY

  @staticmethod
  def Args(parser):
    parser.add_argument(
        '--dependency-images',
        help=(
            'Comma separated list of images containing dependencies for the'
            ' Spark Application.'
        ),
        metavar='IMAGE',
        type=arg_parsers.ArgList(),
    )
