# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command group `gcloud container aws`."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.projects import util


@base.Deprecate(
    is_removed=False,
    warning=(
        "This command is deprecated. See"
        " https://cloud.google.com/kubernetes-engine/multi-cloud/docs/aws/deprecations/deprecation-announcement"
        " for more details."
    ),
    error=(
        "This command has been removed. See"
        " https://cloud.google.com/kubernetes-engine/multi-cloud/docs/aws/deprecations/deprecation-announcement"
        " for more details."
    ),
)
@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class Aws(base.Group):
  """Deploy and manage clusters of machines on AWS for running containers."""

  category = base.COMPUTE_CATEGORY

  def Filter(self, context, args):
    del context, args
